--[[
Traduction de la parole
Ce plugin vous permettra d'utiliser la dictée vocale pour traduire ce que vous allez dire.
Auteur : Abdelkrim BENSAÏD.
Membre du groupe Jieshuo for arabs.
]]

require "import";
import "android.speech.RecognizerIntent";
import "android.net.Uri";
import "java.util.Locale"
import "com.androlua.LuaDialog";
import "com.androlua.Http";
import "android.text.Html";
import "android.speech.RecognitionListener";
import "android.speech.SpeechRecognizer";
import "android.content.Intent";
import "java.io.File";
import "java.io.FileInputStream";
import "java.io.FileWriter";
import "com.androlua.LuaUtil";
import "java.lang.String";
import "java.lang.Integer";
import "java.nio.charset.StandardCharsets";

-- Ceci est la liste des langues sources, que l'on pourras utiliser lors de la dictée vocale.
local langs = {
    "Arabe",
    "Français",
    "Anglais",
    "Espagnol",
    "Allemand"
};
local values = {
    "ar_SA",
    "fr_FR",
    "en_GB",
    "es_ES",
    "de_DE"
}

-- Voici la liste des langues vers lesquelles on va pouvoir traduire.
local targetLangs = {
    "Coréen",
    "Néerlandais",
    "thaïlandais",
    "grec",
    "italien",
    "slovène",
    "japonais",
    "Anglais",
    "arabe",
    "Cantonais",
    "hongrois",
    "estonien",
    "Roumain",
    "Chinois",
    "tchèque",
    "Allemand",
    "Français",
    "finnois",
    "portugais",
    "danois",
    "russe",
    "bulgare",
    "Espagnol",
    "Suèdois",
    "Polonais",
    "Viétnamien",
    "Irlandais" 
};

-- Voici la liste des codes de langues pour le traducteur de Google.
local langCodes = {
    "ko",
    "nl",
    "th",
    "el",
    "it",
    "sl",
    "ja",
    "en",
    "ar",
    "yue",
    "hu",
    "et",
    "ro",
    "zh",
   "cs",
    "de",
    "fr",
    "fi",
    "pt",
    "da",
    "ru",
    "bg",
    "es",
    "sv",
    "pl",
    "vi",
    "ga"
};

local inputText = service.getEditText();

-- Cette fonction permet d'extraire un paramètre d'un fichier conservé dans le dossier du plugin.
function getPluginConfig(fileObject)
    local inputStream = FileInputStream(fileObject);
    local value = LuaUtil.readAll(inputStream);
    value = String(value, StandardCharsets.UTF_8);
    value  = tostring(value);
    return value
end

-- Cette fonction permet d'exécuter une traduction en utilisant Google Translate.
function translateWithGoogle(langTo, textToTranslate)
    local defaultLang = Locale.getDefault().getLanguage();
    local translatedText = "";
    local pattern = '<div class="result%-container">([^<]+)</div>';
    local url = "https://translate.google.com/m?sl=auto&hl=" .. defaultLang .. "&tl=" .. langTo .. "&q=" .. Uri.encode(textToTranslate)
    Http.get(url, function(status, result)
        if status == 200 then
            translatedText = result:match(pattern);
            translatedTexts = Html.fromHtml(translatedText).toString();
            service.insertText(inputText, translatedText);
            service.postSpeak(200, translatedText);
        end
    end);
end

local translateFromFilePath = service.getLuaExtDir() .. File.separator .. "plugins" .. File.separator .. "Traduction de la parole" .. File.separator .. "translate_From";
local translateFromFile = File(translateFromFilePath);
local translateToFilePath = service.getLuaExtDir() .. File.separator .. "plugins" .. File.separator .. "Traduction de la parole" .. File.separator .. "translate_to";
local translateToFile = File(translateToFilePath);

-- Cette fonction permet de gérer la dictée vocale qui sera aussi la langue source pour la traduction.
function voiceTranslation(activity, langFrom, langTo)
    local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, langFrom);
    local recognitionListener = RecognitionListener {
        onResults = function(params)
            -- Ici, nous extrayons le texte que nous avons dicté.
            local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            -- Nous attribuons un objet au texte que nous avons dicté.
            local text = tostring(results[0]);
            translateWithGoogle(langTo, text);
        end,
    }
    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
end

function setTranslation(langFrom)
    local dlg2=LuaDialog()
    dlg2.setTitle("Sélectionnez la langue vers laquelle vous souhaitez traduire");
    dlg2.setMessage([[Choisissez la langue vers laquelle vous souhaitez traduire.
    Cette boîte de dialogue s'ouvre uniquement lorsque le plugin est exécuté pour la première fois dans un champ de saisie, elle s'ouvre toujours lorsque le focus est dans une fenêtre qui ne contient pas de champ de saisie.]]);
    dlg2.setItems(targetLangs);
    dlg2.show();
    dlg2.onItemClick=function(l,v,p,i)
        dlg2.dismiss();
        task(50 , function()
            local source = "";
            for k, v in ipairs(values) do
                if v == langFrom then
                    source = langs[k]
                end
            end
            local langTo = langCodes[i];
            local writer = FileWriter(translateToFilePath);
            writer.write(langTo);
            writer.close();
            if inputText then
                translateTo = getPluginConfig(translateToFile);
                voiceTranslation(this, langFrom, langTo);
                return
            end
            service.speak(source .. " a été choisie comme langue source, " .. targetLangs[i] .. " a été choisie comme langue cible, dirigez-vous vers une fenêtre comportant un champ de saisie pour dicter votre texte");
        end);
    end
    return
end

if not inputText or not translateToFile.exists() then
    dlg=LuaDialog()
    dlg.setTitle("Choisissez la langue à partir de laquelle la traduction sera effectuée");
    dlg.setMessage([[Choisissez la langue source que vous souhaitez utiliser pour qu'elle puisse être traduite.
    Cette boîte de dialogue s'ouvre uniquement lorsque le plugin est exécuté pour la première fois dans un champ de saisie, elle s'ouvre toujours lorsque le focus est dans une fenêtre qui ne contient pas de champ de saisie.]]);
    dlg.setItems(langs);
    dlg.show();
    dlg.onItemClick=function(l,v,p,i)
        dlg.dismiss();
        task(50 , function()
            local langFrom = values[i];
            local writer = FileWriter(translateFromFilePath);
            writer.write(langFrom);
            writer.close();
           setTranslation(langFrom);
        end);
    end
    return
end

if translateToFile.exists() then
    translateTo = getPluginConfig(translateToFile);
    if translateFromFile.exists() then
        translateFrom = getPluginConfig(translateFromFile);
        voiceTranslation(this, translateFrom, translateTo);
    end
end
return true;